/* $Id: rc5.c 11 2009-10-14 06:21:07Z andreas $ */

/*
 * Copyright (c) 2006-2009 Andreas Schroeder <andreas@a-netz.de>
 *
 * Implement functions of an infrared remote control and RC5 protocol with an
 * Atmel AVR controller.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <avr/io.h>
#include <avr/interrupt.h>

#include "rc5.h"
#include "rc5_codes.h"

/* RC5 device code to send */
#define RC5_DEVICE_CODE       RC5_DEV_EXPERIMENTAL

/*
 * Maximum bit count before transmission is finished. Minimum (as
 * protocol standard says) is 64. Higher values yield a lower repetition rate.
 */
#define MAX_BITCOUNT 128

/*
 * The template contains the start bits and the rc5 device code. It is combined
 * with the rc5 command code of the telegram to be sent.
 */
#define RC5_TEMPLATE (0x3000 | (RC5_DEVICE_CODE << 6))

/*
 * modulation square wave generation:
 * compare output mode: toggle OC0A on compare match
 * if COM0Ax are *all* unset, the waveform generation is disabled and
 * the normal PORTx value is seen on the OC0A pin.
 */

/* Enable the 36kHz waveform generation */
#define ENABLE_MODULATION() (TCCR0A |= _BV(COM0A0))
/* Disable the 36kHz waveform generation */
#define DISABLE_MODULATION() (TCCR0A &= ~_BV(COM0A0))

/* Enable bit timer */
#define ENABLE_BIT_TIMER() (TCCR1B |= _BV(CS10))
/* Disable bit timer */
#define DISABLE_BIT_TIMER() (TCCR1B &= ~_BV(CS10))

/*
 * Contains the data of the currently sent rc5 telegram.
 */
volatile uint16_t rc5data = 0;

/*
 * The bitcounter tells which bit is currently sent. It counts up to 
 * MAX_BITCOUNT (must be more that 14 to ensure that all bits are transmitted).
 */
volatile uint8_t bitcounter = MAX_BITCOUNT;
/* 
 * The haltbit counter toggles between 0 and 1 and determines if the first or
 * second part of a bit is sent.
 */
volatile uint8_t halfbitcounter;

/*
 * Initialize the rc5 hardware.
 */
void rc5_init(void) {
  // setup waveform generation to 36kHz on timer 0
  TCCR0A |= _BV(WGM01); // Clear timer on compare match
  TCCR0B |= _BV(CS00); // timer source is system clock
  
  // verify correct frequency or use a crystal. The internal rc oscillator is 
  // not really accurate
  //OCR0A = 107; // 36kHz @ 8MHz
  OCR0A = 105; // 36kHz @ 7,77Mhz

  // this port drives the infrared led
  DDRB |= _BV(PB2); // port b.2 as output
  PORTB &= ~_BV(PB2); // pb2 is lo

  // setup half-bit clock (555Hz) on timer1
  TCCR1A = 0;
  TCCR1B = _BV(WGM12); // Clear timer on compare match
  // verify correct frequency or use a crystal. The internal rc oscillator is 
  // not really accurate
  OCR1A = 7049; // ~ 889us @ 8MHz

  TIMSK |= _BV(OCIE1A); // output compare interrupt enable 1a
}


/*
 * Start rc5 transmission. 
 */
void rc5_send(rc5_cmd_t cmd) {
  // assemble rc5 packet from TEMPLATE (2 start bits and device identifier)
  // and 6 command bits
  rc5data = (RC5_TEMPLATE | (cmd & 0x3f));

  // activate timer 1 (half bit timer) to write out bitstream
  bitcounter = 0;
  halfbitcounter = 0;

  // now start sending by enabling the half-bit timer.
  ENABLE_BIT_TIMER();
}


/*
 * Timer 1 compare A interrupt handler. 
 *
 * The handler is called after every half-bit send. 
 */
ISR(TIMER1_COMPA_vect) {
  uint8_t bit = ((rc5data & 0x2000) >> 13);
  uint8_t half = (bit ^ ((halfbitcounter == 0) ? 1 : 0));

  // transmit 14 bit rc5 packet
  if(half && bitcounter < 14) {
    ENABLE_MODULATION();
  } else {
    DISABLE_MODULATION();
  }

  if(bitcounter >= MAX_BITCOUNT) {
    // inter-telegram time has expired, so stop the bit timer. We have completed
    // sending the last rc5 telegram.
    DISABLE_BIT_TIMER();
    rc5_on_send_complete();
  } else if(bitcounter == 14) {
    // a complete rc5 telegram has been transmitted. We disable modulation but
    // keep timer interrupts enabled to wait for the inter-telegram time to
    // expire
    DISABLE_MODULATION();
    rc5_on_telegram_complete();
  }

  if(halfbitcounter == 0) {
    // the first half-bit has been sent, now save state to sent the second 
    // half-bit
    halfbitcounter++;
  } else {
    // a complete bit has been sent. Save state to begin sending the next
    // bit in the next cycle. 
    halfbitcounter = 0;
    bitcounter++;
    rc5data <<= 1;
  }
}