/* $Id: rc.c 11 2009-10-14 06:21:07Z andreas $ */

/*
 * Copyright (c) 2006-2009 Andreas Schroeder <andreas@a-netz.de>
 *
 * Implement functions of an infrared remote control and RC5 protocol with an
 * Atmel AVR controller.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @mainpage
 * The remote-control project contains source code to enable an Atmel AVR 
 * conroller to send RC5 telegrams over infrared to a suitable receiver.
 * See http://www.a-netz.de for schematics and more info.
 */

/** 
 * @file
 * This file contains the program main entry piont and the glue logic to 
 * send a RC5 telegram after a button has been pressed.
 *
 * After reset the hardware is initialized, the external interrupt for the 
 * buttons is enabled and the controller is set to sleep mode. After a key press
 * an interrupt is generated and the controller wakes up. It disables the 
 * buttons and sends the RC5 telegram. When the sending is finished the buttons
 * are re-enabled and the controller is put to sleep again.
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "led.h"
#include "buttons.h"
#include "rc5.h"


/**
 * External interrupt 0 handler. 
 * 
 * This is connected to the push buttons. They are pulling the interrupt pin
 * to low to generate an interrupt.
 */
ISR(INT0_vect) {
  rc5_cmd_t cmd;

  if(rc5_get_cmd(&cmd) == 0) {
    led_set(1);

    sleep_disable();
    DISABLE_BUTTONS();

    rc5_send(cmd);
  }
}

/**
 * Callback function from the rc5 handler. Called after a telegram has been 
 * sent.
 */
void rc5_on_send_complete(void) {
  ENABLE_BUTTONS();

  sleep_enable();
}

/**
 * Callback function from the rc5 handler. 
 */
void rc5_on_telegram_complete(void) {
  led_set(0);
}

/**
 * Program main entry point.
 */
int main(void) {
  // setup rc oscillator clock divider to div 1
  // (=> 8MHz with internal RC oscollator)
  CLKPR = _BV(CLKPCE); // enable clock prescale change
  CLKPR = 0; // div1 => 8 MHz
  // save energy, disable analog comaprator
  ACSR |= _BV(ACD);

  //initialize function modules
  rc5_init();
  buttons_init();
  led_init();

  // enable external interrupt
  ENABLE_BUTTONS();

  sei();
  // set sleep mode to power down mode (i.e. all functions except external
  // level (!) interrupts are disabled. ATtiny2313 needs ~ 0.5uA then.
  set_sleep_mode( _BV(SM0) | _BV(SM1) );

  // enable sleep mode
  sleep_enable();

  for(;;) {
    // sleep and wait for external interrupt (from button press)
    sleep_cpu();
  }
}

